/* Process model C form file: fy_pc_fifo_1.pr.c */
/* Portions of this file Copyright (C) MIL 3, Inc. 1992 */



/* OPNET system definitions */
#include <opnet.h>
#include "fy_pc_fifo_1.pr.h"
FSM_EXT_DECS



/* Header block */
#define ARR_Q_EMPTY  (op_intrpt_type () == OPC_INTRPT_STRM && \
                      op_stat_local_read(0) == 0.0)
#define ARR_Q_BUSY   (op_intrpt_type () == OPC_INTRPT_STRM && \
                      op_stat_local_read(0) == 1.0)
#define SERVICE      op_intrpt_type () == OPC_INTRPT_STAT

/*#define SIM_END      (op_intrpt_type () == OPC_INTRPT_ENDSIM)*/

#define QUEUE_DEPTH		0
#define ARRIVAL_PT_Q_LENGTH_STAT 1
#define LATEST_Q_LENGTH_STAT 2



/* State variable definitions */
typedef struct
	{
	FSM_SYS_STATE
	double                  		sv_thru;
	double                  		sv_NoOfPacketsInQueue;
	} fy_pc_fifo_1_state;

#define pr_state_ptr            		((fy_pc_fifo_1_state*) SimI_Mod_State_Ptr)
#define thru                    		pr_state_ptr->sv_thru
#define NoOfPacketsInQueue      		pr_state_ptr->sv_NoOfPacketsInQueue




/* Process model interrupt handling procedure */



void
fy_pc_fifo_1 ()
	{
	packet          *pkptr;
	
	


	FSM_ENTER (fy_pc_fifo_1)

	FSM_BLOCK_SWITCH
		{
		/*---------------------------------------------------------*/
		/** state (INS_TAIL) enter executives **/
		FSM_STATE_ENTER_FORCED (0, state0_enter_exec, "INS_TAIL")
			{
			/* a new packet has arrived; acquire it */
			pkptr = op_pk_get (op_intrpt_strm ());
			
			/*queue_length = op_subq_stat(0, OPC_QSTAT_PKSIZE);*/
			
			/* update arrival point queue statistics */
			/*op_stat_local_write(ARRIVAL_PT_Q_LENGTH_STAT, queue_length);*/
			
			/* insert the new packet at tail of subqueue 0 */
			if (op_subq_pk_insert (0, pkptr, OPC_QPOS_TAIL) != 
			            OPC_QINS_OK)
			{
				/* if the insertion failed, discard the packet */
				op_pk_destroy (pkptr);
			}
			
			NoOfPacketsInQueue++;
			thru = NoOfPacketsInQueue/op_sim_time();
			op_stat_local_write(QUEUE_DEPTH, thru );
			
			/* update latest queue length statistic */
			/*queue_length = op_subq_stat(0, OPC_QSTAT_PKSIZE);
			op_stat_local_write(LATEST_Q_LENGTH_STAT, queue_length);*/
			
			}


		/** state (INS_TAIL) exit executives **/
		FSM_STATE_EXIT_FORCED (0, state0_exit_exec, "INS_TAIL")
			{
			}


		/** state (INS_TAIL) transition processing **/
		FSM_TRANSIT_FORCE (2, state2_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (SND_HD) enter executives **/
		FSM_STATE_ENTER_FORCED (1, state1_enter_exec, "SND_HD")
			{
			/* a request has been made to access the queue */
			/* check if its empty */
			if (!op_subq_empty (0))
			{
				/* access the first packet in the subqueue */
				pkptr = op_subq_pk_remove (0, OPC_QPOS_HEAD);
			
				/* forward it to the destination */
				/* causing a stream interrupt */
				/* because we are using stat interrupt not access */
				op_pk_send (pkptr, 0);
			
				NoOfPacketsInQueue--;
				thru = NoOfPacketsInQueue/op_sim_time();
				op_stat_local_write(QUEUE_DEPTH, thru );
			}
			}


		/** state (SND_HD) exit executives **/
		FSM_STATE_EXIT_FORCED (1, state1_exit_exec, "SND_HD")
			{
			}


		/** state (SND_HD) transition processing **/
		FSM_TRANSIT_FORCE (2, state2_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (BRANCH) enter executives **/
		FSM_STATE_ENTER_UNFORCED (2, state2_enter_exec, "BRANCH")
			{
			
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (5,fy_pc_fifo_1)


		/** state (BRANCH) exit executives **/
		FSM_STATE_EXIT_UNFORCED (2, state2_exit_exec, "BRANCH")
			{
			}


		/** state (BRANCH) transition processing **/
		FSM_INIT_COND (SERVICE)
		FSM_TEST_COND (ARR_Q_BUSY)
		FSM_TEST_COND (ARR_Q_EMPTY)
		FSM_TEST_LOGIC ("BRANCH")

		FSM_TRANSIT_SWITCH
			{
			FSM_CASE_TRANSIT (0, 1, state1_enter_exec, ;)
			FSM_CASE_TRANSIT (1, 0, state0_enter_exec, ;)
			FSM_CASE_TRANSIT (2, 3, state3_enter_exec, ;)
			}
		/*---------------------------------------------------------*/



		/** state (SND_ARR) enter executives **/
		FSM_STATE_ENTER_FORCED (3, state3_enter_exec, "SND_ARR")
			{
			/* there is an arrival and the server is free */
			/* get the packet pointer and send the packet */
			/* update  queue statistics */
			
			/* update queue  statistics */
			/*op_stat_local_write(ARRIVAL_PT_Q_LENGTH_STAT, 0);
			op_stat_local_write(LATEST_Q_LENGTH_STAT, 0);*/
			
			NoOfPacketsInQueue++;
			thru = NoOfPacketsInQueue/op_sim_time();
			op_stat_local_write(QUEUE_DEPTH, thru );
			
			pkptr = op_pk_get (op_intrpt_strm ());
			op_pk_send (pkptr, 0);
			}


		/** state (SND_ARR) exit executives **/
		FSM_STATE_EXIT_FORCED (3, state3_exit_exec, "SND_ARR")
			{
			}


		/** state (SND_ARR) transition processing **/
		FSM_TRANSIT_FORCE (2, state2_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (INIT) enter executives **/
		FSM_STATE_ENTER_FORCED (4, state4_enter_exec, "INIT")
			{
			}


		/** state (INIT) exit executives **/
		FSM_STATE_EXIT_FORCED (4, state4_exit_exec, "INIT")
			{
			}


		/** state (INIT) transition processing **/
		FSM_TRANSIT_FORCE (2, state2_enter_exec, ;)
		/*---------------------------------------------------------*/



		}


	FSM_EXIT (4,fy_pc_fifo_1)
	}




void
fy_pc_fifo_1_svar (prs_ptr,var_name,var_p_ptr)
	fy_pc_fifo_1_state		*prs_ptr;
	char			*var_name, **var_p_ptr;
	{

	FIN (fy_pc_fifo_1_svar (prs_ptr))

	*var_p_ptr = VOS_NIL;
	if (Vos_String_Equal ("thru" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_thru);
	if (Vos_String_Equal ("NoOfPacketsInQueue" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_NoOfPacketsInQueue);

	FOUT;
	}




void
fy_pc_fifo_1_diag ()
	{
	packet          *pkptr;
	
	

	FIN (fy_pc_fifo_1_diag ())


	FOUT;
	}




void
fy_pc_fifo_1_terminate ()
	{
	packet          *pkptr;
	
	

	FIN (fy_pc_fifo_1_terminate ())


	FOUT;
	}



Compcode
fy_pc_fifo_1_init (pr_state_pptr)
	fy_pc_fifo_1_state			**pr_state_pptr;
	{
	static VosT_Cm_Obtype	obtype = OPC_NIL;

	FIN (fy_pc_fifo_1_init (pr_state_pptr))

	if (obtype == OPC_NIL)
		{
		if (Vos_Catmem_Register ("proc state vars (fy_pc_fifo_1)",
			sizeof (fy_pc_fifo_1_state), Vos_Nop, &obtype) == VOSC_FAILURE)
			FRET (OPC_COMPCODE_FAILURE)
		}

	if ((*pr_state_pptr = (fy_pc_fifo_1_state*) Vos_Catmem_Alloc (obtype, 1)) == OPC_NIL)
		FRET (OPC_COMPCODE_FAILURE)
	else
		{
		(*pr_state_pptr)->current_block = 8;
		FRET (OPC_COMPCODE_SUCCESS)
		}
	}



